<?xml version="1.0" encoding="US-ASCII"?>
<?xml-stylesheet type="text/xsl" href="../xslstyle/xslstyle-docbook.xsl"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                xmlns:xs="http://www.CraneSoftwrights.com/ns/xslstyle"
                xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                xmlns:s="urn:x-CraneSoftwrights:ss:sch2html"
                exclude-result-prefixes="xsd xs sch s"
                version="2.0">

<xs:doc info="$Id: Crane-sch2html.xsl,v 1.24 2008/07/04 21:56:25 gkholman Exp $"
        filename="Crane-sch2html.xsl" global-ns="xs" internal-ns="s" 
        vocabulary="DocBook">
  <xs:title>Schematron rules to HTML rendering</xs:title>
  <para>
    This renders a summary of the messages that may be reported from a
    Schematron schema.  See the associated <literal>sch2html.bat</literal>
    for an example invocation from a Windows-based system.
  </para>
  <programlisting>
Copyright (C) - Crane Softwrights Ltd. 

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, 
  this list of conditions and the following disclaimer. 
- Redistributions in binary form must reproduce the above copyright notice, 
  this list of conditions and the following disclaimer in the documentation 
  and/or other materials provided with the distribution. 
- The name of the author may not be used to endorse or promote products 
  derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN 
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2002-12-16 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5
  </programlisting>
</xs:doc>

<xs:doc>
  <xs:title>Invocation</xs:title>
  <para>
    Using stylesheet association one can reference the stylesheet at the top
    of the Schematron schema for automatic invocation by an aware processor: 
  </para>
  <blockquote>
    <programlisting>
<![CDATA[<?xml-stylesheet type="text/xsl" href="Crane-sch2html.xsl"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron" ...]]></programlisting>
  </blockquote>
  <para>
    Alternatively, one can invoke the stylesheet with a copy of the input
    filename as a passed parameter.
  </para>
  <para>
    Note that for Windows systems the Windows Manager icon for the supplied 
    <literal>Crane-sch2html.bat</literal> file can be used in a drag-and-drop
    fashion to create an HTML file without using the command prompt.  Just
    drop the icon for the Schematron schema on top of the icon for the batch
    file.
  </para>
</xs:doc>

<xs:param ignore-ns="yes">
  <para>
    This argument passed during invocation is the filename of the input file used for reporting.
  </para>
  <para>
    If this is empty, the stylesheet will attempt to figure it out for itself.
  </para>
  <para>
    If the stylesheet cannot do so, the arbitrary string
    <literal>Input Schematron file</literal> is used.
  </para>
</xs:param>
<xsl:param name="inFileName" as="xsd:string" select="''"/>

<!--========================================================================-->
<xs:doc>
  <xs:title>Stylesheet logic</xs:title>
  <para>
    Note that throughout the stylesheet where you see a select with
    <literal>"/*/sch:include/@href"</literal> in fact it should work with
    <literal>"//sch:include/@href"</literal> but apparently that doesn't in
    Internet Explorer.
  </para>
</xs:doc>

<xs:variable>
  <para>The calculation of the filename for use in reporting.</para>
</xs:variable>
<xsl:variable name="s:inFileName" as="xsd:string">
  <xsl:choose>
    <xsl:when test="string($inFileName)">
      <!--supplied-->
      <xsl:value-of select="string($inFileName)"/>
    </xsl:when>
    <xsl:when test="function-available('document-uri')">
      <!--calculatable-->
      <xsl:value-of select="document-uri(/)"/>
    </xsl:when>
    <xsl:otherwise>Input Schematron file</xsl:otherwise>
  </xsl:choose>
</xsl:variable>

<xs:variable>
  <para>
    The input document expressed as a document tree using the document()
    function, because "document('',/)" may be different than "/".
  </para>
</xs:variable>
<xsl:variable name="s:first" select="document('',/)" as="document-node()"/>

<xs:template>
  <para>Getting started</para>
</xs:template>
<xsl:template match="/">
  <xsl:if test="not(sch:*)">
    <!--problem-->
    <xsl:message terminate="yes">
      <xsl:text>The input file is not an ISO Schematron schema.</xsl:text>
    </xsl:message>
  </xsl:if>
  <html>
    <head>
      <title><xsl:value-of select="$s:inFileName"/> - Schematron rules</title>
    </head>
    <body>
      <!--handle the main input the same way all others are handled-->
      <xsl:call-template name="s:find-all">
        <xsl:with-param name="hrefs" 
                        select="document('',/)/*/sch:include/@href"/>
      </xsl:call-template>
      <!--advertising-->
      <p align="right">
        <small>
          <xsl:text/>Report created by <samp>Crane-sch2html.xsl</samp>
          <xsl:text> $Revision: 1.24 $ </xsl:text>
          <a href="http://www.CraneSoftwrights.com/links/res-dev.htm"
             target="_blank">
            <xsl:text>Crane Softwrights Ltd.</xsl:text>
          </a>
        </small>
      </p>
    </body>
  </html>
</xsl:template>

<xs:template>
  <para>
    Determine all of the fragments in the entire schema, relying on the
    XPath union of node sets not duplicating the same node.
  </para>
  <xs:param name="hrefs">
    <para>All of the include directive attributes found so far.</para>
  </xs:param>
</xs:template>
<xsl:template name="s:find-all">
  <xsl:param name="hrefs" as="attribute(href)*" required="yes"/>
  <!--add all of the included fragments from the current set-->
  <xsl:variable name="new-hrefs" select="$hrefs | 
                                         document($hrefs)/*//sch:include/@href"
                as="attribute(href)*"/>
  <xsl:choose>
    <xsl:when test="count($hrefs) != count($new-hrefs)">
      <!--still finding new includes that need to be de-referenced-->
      <xsl:call-template name="s:find-all">
        <xsl:with-param name="hrefs" select="$new-hrefs"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <!--found all possible include files throughout the report-->
      <!--report on the base file first-->
      <xsl:call-template name="s:report-one">
        <xsl:with-param name="this" select="$s:first"/>
        <xsl:with-param name="hrefs" select="$new-hrefs"/>
      </xsl:call-template>
      <!--then report on all of the included files in sorted href order-->
      <xsl:for-each select="$new-hrefs">
        <xsl:sort select="."/>
        <xsl:call-template name="s:report-one">
          <xsl:with-param name="this" select="document(.)"/>
          <xsl:with-param name="hrefs" select="$new-hrefs"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xs:template>
  <para>
    Do all processing for one fragment.
  </para>
  <xs:param name="this">
    <para>The document fragment being processed</para>
  </xs:param>
  <xs:param name="hrefs">
    <para>All of the include directive attributes.</para>
  </xs:param>
</xs:template>
<xsl:template name="s:report-one">
  <xsl:param name="this" as="document-node()" required="yes"/>
  <xsl:param name="hrefs" as="attribute(href)*" required="yes"/>
  <!--determine the fragment number-->
  <xsl:variable name="fragment">
    <xsl:choose>
      <xsl:when test="generate-id($this)=generate-id($s:first)">1</xsl:when>
      <xsl:otherwise><xsl:value-of select="position()+1"/></xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <!--anchor for hyperlinks-->
  <a name="{generate-id($this)}"/>
  <!--title the current fragment-->
  <xsl:choose>
    <xsl:when test="generate-id($this)=generate-id($s:first)">
      <h1>
        <xsl:call-template name="s:fragment-title">
          <xsl:with-param name="this" select="$this"/>
          <xsl:with-param name="hrefs" select="$hrefs"/>
        </xsl:call-template>
      </h1>
    </xsl:when>
    <xsl:otherwise>
      <h2>
        <xsl:call-template name="s:fragment-title">
          <xsl:with-param name="this" select="$this"/>
          <xsl:with-param name="hrefs" select="$hrefs"/>
        </xsl:call-template>
      </h2>
    </xsl:otherwise>
  </xsl:choose>
  <!--now report the content of the fragment-->
  <xsl:for-each select="$this">
    <xsl:choose>
      <!--only check on rules, includes and patterns not in a pattern-->
      <xsl:when test="not(//sch:pattern | 
                      (//sch:include|//sch:rule)[not(ancestor::sch:pattern)])">
        <blockquote>
          <p><i>No schematron rules present in this schema fragment.</i></p>
        </blockquote>
      </xsl:when>
      <!--ready to report the items found in this fragment-->
      <xsl:otherwise>
        <xsl:apply-templates select="//sch:pattern | 
                       (//sch:include|//sch:rule)[not(ancestor::sch:pattern)]">
          <xsl:with-param name="hrefs" select="$hrefs"/>
          <xsl:with-param name="fragment" select="string($fragment)"/>
          <xsl:with-param name="pattern" select="string($fragment)"/>
        </xsl:apply-templates>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:for-each>
</xsl:template>

<xs:template>
  <para>Report on an include found in play not in a pattern</para>
  <xs:param name="hrefs">
    <para>All of the include directive attributes.</para>
  </xs:param>
  <xs:param name="fragment">
    <para>The string by which the fragment is referenced.</para>
  </xs:param>
</xs:template>
<xsl:template match="sch:include">
  <xsl:param name="hrefs" as="attribute(href)*" required="yes"/>
  <xsl:param name="fragment" as="xsd:string"/>
  <h3>
    <xsl:value-of select="concat($fragment,'.',position())"/>. Include
    <xsl:for-each select="document(@href)">
      <a href="#{generate-id(.)}">
        <xsl:call-template name="s:fragment-title">
          <xsl:with-param name="this" select="."/>
          <xsl:with-param name="hrefs" select="$hrefs"/>
        </xsl:call-template>
      </a>
    </xsl:for-each>
  </h3>
</xsl:template>

<xs:template>
  <para>Report on a pattern</para>
  <xs:param name="hrefs">
    <para>All of the include directive attributes.</para>
  </xs:param>
  <xs:param name="fragment">
    <para>The string by which the fragment is referenced.</para>
  </xs:param>
</xs:template>
<xsl:template match="sch:pattern">
  <xsl:param name="hrefs" as="attribute(href)*" required="yes"/>
  <xsl:param name="fragment" as="xsd:string"/>
  <!--distinguish the patterns by their identifier-->
  <h3>
    <xsl:value-of select="concat($fragment,'.',position())"/>. Pattern
    <samp>
      <xsl:value-of select="@id"/>
    </samp>
  </h3>
  <!--report the messages-->
  <blockquote>
    <xsl:apply-templates select=".//sch:rule | .//sch:include" mode="s:in-pat">
      <!--include the ordinal of the rule-->
      <xsl:with-param name="pattern" 
                      select="concat($fragment,'.',position())"/>
      <xsl:with-param name="hrefs" select="$hrefs"/>
    </xsl:apply-templates>
  </blockquote>
</xsl:template>

<xs:template>
  <para>Report on an include found in play in a pattern</para>
  <xs:param name="hrefs">
    <para>All of the include directive attributes.</para>
  </xs:param>
  <xs:param name="pattern">
    <para>The string by which the pattern is referenced.</para>
  </xs:param>
</xs:template>
<xsl:template match="sch:include" mode="s:in-pat">
  <xsl:param name="pattern" as="xsd:string"/>
  <xsl:param name="hrefs" as="attribute(href)*" required="yes"/>
  <h4>
    <xsl:value-of select="concat($pattern,'.',position())"/>. Include
    <xsl:for-each select="document(@href)">
      <a href="#{generate-id(.)}">
        <xsl:call-template name="s:fragment-title">
          <xsl:with-param name="this" select="."/>
          <xsl:with-param name="hrefs" select="$hrefs"/>
        </xsl:call-template>
      </a>
    </xsl:for-each>
  </h4>
</xsl:template>

<xs:template>
  <para>
    Report on a rule when in a pattern.
  </para>
  <xs:param name="pattern">
    <para>The ordinal of the pattern being reported.</para>
  </xs:param>
</xs:template>
<xsl:template match="sch:rule" mode="s:in-pat">
  <xsl:param name="pattern" as="xsd:string"/>
  <!--distinguish the rules by their context-->
  <h4>
    <xsl:value-of select="concat($pattern,'.',position())"/>. Rule
    <samp>
      <xsl:value-of select="@context"/>
    </samp>
  </h4>
  <xsl:call-template name="s:rule-report">
    <xsl:with-param name="pattern" select="$pattern"/>
  </xsl:call-template>
</xsl:template>

<xs:template>
  <para>Report on a rule when not in a pattern.</para>
  <xs:param name="pattern">
    <para>The ordinal of the pattern being reported.</para>
  </xs:param>
</xs:template>
<xsl:template match="sch:rule">
  <xsl:param name="pattern" as="xsd:string"/>
  <!--distinguish the rules by their context-->
  <h3>
    <xsl:value-of select="concat($pattern,'.',position())"/>. Rule
    <samp>
      <xsl:value-of select="@context"/>
    </samp>
  </h3>
  <xsl:call-template name="s:rule-report">
    <xsl:with-param name="pattern" select="$pattern"/>
  </xsl:call-template>
</xsl:template>

<xs:template>
  <para>Report common information for all rules.</para>
  <xs:param name="pattern">
    <para>The ordinal of the pattern being reported.</para>
  </xs:param>
</xs:template>
<xsl:template name="s:rule-report">
  <xsl:param name="pattern" as="xsd:string"/>
  <blockquote>
    <p>
      <xsl:apply-templates select="sch:assert | sch:test">
        <!--include the ordinal of the rule-->
        <xsl:with-param name="rule" select="concat($pattern,'.',position())"/>
      </xsl:apply-templates>
    </p>
  </blockquote>
</xsl:template>

<xs:template>
  <para>Report a message's content</para>
  <xs:param name="rule">
    <para>The ordinal of the rule being reported.</para>
  </xs:param>
</xs:template>
<xsl:template match="sch:assert | sch:test">
  <xsl:param name="rule" as="xsd:string"/>
  <p>
    <!--report the ordinal of the message within the rule-->
    <xsl:value-of select="concat($rule,'.',position())"/>:
    <samp>
      <xsl:apply-templates/>
    </samp>
  </p>
</xsl:template>

<xs:template>
  <para>Report all Schematron vocabulary constructs in italics.</para>
  <para>Include the "select" attribute where present.</para>
</xs:template>
<xsl:template match="sch:*">
  <i>
    <xsl:value-of select="local-name(.)"/>
    <xsl:for-each select="@select">
      <xsl:text/>(<xsl:value-of select="."/>)<xsl:text/>
    </xsl:for-each>
  </i>
</xsl:template>

<xs:template>
  <para>
    Generate the title for the given fragment
  </para>
  <xs:param name="this">
    <para>The document fragment being processed</para>
  </xs:param>
  <xs:param name="hrefs">
    <para>All of the pointers to document fragments.</para>
  </xs:param>
</xs:template>
<xsl:template name="s:fragment-title">
  <xsl:param name="this" as="document-node()" required="yes"/>
  <xsl:param name="hrefs" as="attribute(href)*" required="yes"/>
  <!--calculate the title in order to know what to do-->
  <xsl:variable name="title">
    <xsl:for-each select="$hrefs">
      <xsl:sort select="."/>
      <xsl:if test="generate-id(document(.))=generate-id($this)">
        <xsl:value-of select="position() + 1"/>
        <xsl:text>. </xsl:text>
        <samp>
          <xsl:value-of select="."/>
        </samp>
      </xsl:if>
    </xsl:for-each>
  </xsl:variable>
  <xsl:choose>
    <xsl:when test="not(string($title))">
      <!--the fragment isn't being included, so must be the top-most-->
      <xsl:text>1. </xsl:text>
      <xsl:value-of select="$s:inFileName"/> - Schematron rules<xsl:text/>
    </xsl:when>
    <xsl:otherwise>
      <!--the fragment is being included somewhere-->
      <xsl:copy-of select="$title"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

</xsl:stylesheet>